// Copyright (c) 2015-present, salesforce.com, inc. All rights reserved
// Licensed under BSD 3-Clause - see LICENSE.txt or git.io/sfdc-license

const express = require('express');
const path = require('path');
const Either = require('data.either');

const rootPath = path.resolve.bind(path, __dirname, '../');

module.exports = ({ publicPath = {}, cssUrl, scriptUrl, scriptPath }) => {
  const app = express();

  const assetUrl = (version, url) =>
    `https://archive-${version}.lightningdesignsystem.com${url}`;

  const getCssUrl = version =>
    Either.fromNullable(cssUrl).fold(
      () =>
        assetUrl(
          version,
          '/assets/styles/salesforce-lightning-design-system.min.css'
        ),
      url => url
    );

  const getScriptUrl = version =>
    Either.fromNullable(scriptUrl).fold(
      () => assetUrl(version, '/assets/__internal/slds.umd.js'),
      url => url
    );

  const iframeTemplate = (version, { scripts, css }) =>
    `
    <html lang="en">
      <head>
        <meta charSet="utf-8" />
        <meta name="viewport" content="width=device-width, minimum-scale=1" />
        <link type="text/css" rel="stylesheet" href="${getCssUrl(version)}" />
        ${css || ''}
      </head>
      <body>
        <div id="previewer" style="padding: 2rem;"></div>
        ${scripts || ''}
      </body>
    </html>
    `;

  app.get('/env.js', (req, res) =>
    res.send(`
      var SLDS_BUNDLE_URL = "${scriptUrl}";
    `)
  );

  app.get('/:version/slds.js', (req, res) =>
    res.redirect(getScriptUrl(req.params.version))
  );

  app.get('/:version/preview', (req, res) => {
    return res.sendFile(rootPath('public/index.html'));
  });

  app.get('/:version/__DOC_FRAME__', (req, res) => {
    res.send(
      iframeTemplate(req.params.version, {
        scripts: `<script src="${getScriptUrl()}"></script>`
      })
    );
  });

  app.get('/:version/__IFRAME__', (req, res) => {
    res.send(
      iframeTemplate(req.params.version, {
        scripts: '<script src="/__ASSETS__/scripts/bundle/iframe.js"></script>'
      })
    );
  });

  Object.keys(publicPath).forEach(key => {
    publicPath[key].forEach(p => {
      app.use(key, express.static(p));
    });
  });
  if (process.env.SLDS_PREVIEWER === 'development') {
    const webpack = require('webpack');
    const webpackMiddleware = require('webpack-dev-middleware');
    const webpackConfig = require('../webpack.config');
    const compiler = webpack(webpackConfig);
    app.use(
      webpackMiddleware(compiler, {
        noInfo: true,
        publicPath: webpackConfig.output.publicPath,
        quiet: false
      })
    );
  }

  app.use(express.static(rootPath('public')));

  const localDesignSystemAssetPath = path.resolve(
    path.dirname(require.resolve('@salesforce-ux/design-system/package.json')),
    'assets'
  );

  app.use('/__ASSETS__', express.static(localDesignSystemAssetPath));

  if (process.env.NODE_ENV === 'production') {
    app.use('/assets', express.static(localDesignSystemAssetPath));
  }

  return app;
};
